import os
import unittest
from pathlib import Path
from unittest import mock

from checkov.bitbucket.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.runner_filter import RunnerFilter
from checkov.bitbucket.registry import registry


class TestRunnerValid(unittest.TestCase):

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.BITBUCKET_CONFIGURATION)

    @mock.patch.dict(os.environ, {"CKV_BITBUCKET_CONFIG_FETCH_DATA": "False", "PYCHARM_HOSTED": "1",
                                  "BITBUCKET_REPO_FULL_NAME": "shaharsamira/terragoat2"}, clear=True)
    def test_runner_object_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "bitbucket_conf", "fail")
        runner = Runner()
        runner.bitbucket.bitbucket_conf_dir_path = valid_dir_path

        checks = ["CKV_BITBUCKET_1"]
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(checks=checks)
        )
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(report.skipped_checks, [])

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "bitbucket_conf", "fail")
        runner = Runner()
        filter = RunnerFilter(framework=['bitbucket_configuration'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.BITBUCKET_CONFIGURATION: Severities[BcSeverities.OFF]}
        runner.bitbucket.bitbucket_conf_dir_path = valid_dir_path

        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=filter
        )
        summary = report.get_summary()

        assert summary["passed"] == 0
        assert summary["failed"] == 0
        assert summary["skipped"] == 0
        assert summary["parsing_errors"] == 0

    @mock.patch.dict(os.environ, {"CKV_BITBUCKET_CONFIG_FETCH_DATA": "False", "PYCHARM_HOSTED": "1"}, clear=True)
    def test_runner_object_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources", "bitbucket_conf", "pass")
        runner = Runner()
        runner.bitbucket.bitbucket_conf_dir_path = valid_dir_path

        checks = ["CKV_BITBUCKET_1"]
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(checks=checks)
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 1)
        self.assertEqual(report.skipped_checks, [])

    @mock.patch.dict(os.environ, {"CKV_BITBUCKET_CONFIG_FETCH_DATA": "False", "PYCHARM_HOSTED": "1"}, clear=True)
    def test_runner_files_ignore(self):
        # given
        test_file = Path(__file__).parent / "resources/bitbucket_conf/pass/branch_restrictions.json"
        checks = ["CKV_BITBUCKET_1"]

        # when
        report = Runner().run(
            files=[str(test_file)],
            runner_filter=RunnerFilter(checks=checks)
        )

        # then
        # even it points to a file with scannable content, it should skip it
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        self.assertEqual(len(report.skipped_checks), 0)


if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.RedshiftClusterLogging import check
from checkov.common.models.enums import CheckResult


class TestRedshiftClusterLogging(unittest.TestCase):

    def test_failure_missing_1(self):
        hcl_res = hcl2.loads("""
        resource "aws_redshift_cluster" "default" {
          cluster_identifier = "tf-redshift-cluster"
          database_name      = "mydb"
          master_username    = "foo"
          master_password    = "Mustbe8characters"  # checkov:skip=CKV_SECRET_6 test secret
          node_type          = "dc1.large"
          cluster_type       = "single-node"
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_missing_2(self):
        hcl_res = hcl2.loads("""
        resource "aws_redshift_cluster" "default" {
          cluster_identifier = "tf-redshift-cluster"
          database_name      = "mydb"
          master_username    = "foo"
          master_password    = "Mustbe8characters"
          node_type          = "dc1.large"
          cluster_type       = "single-node"
          logging {
            somethingelse = "true"
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


    def test_failure_false(self):
        hcl_res = hcl2.loads("""
        resource "aws_redshift_cluster" "default" {
          cluster_identifier = "tf-redshift-cluster"
          database_name      = "mydb"
          master_username    = "foo"
          master_password    = "Mustbe8characters"
          node_type          = "dc1.large"
          cluster_type       = "single-node"
          logging {
            enable = 0   
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)
  
    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "aws_redshift_cluster" "default" {
          cluster_identifier = "tf-redshift-cluster"
          database_name      = "mydb"
          master_username    = "foo"
          master_password    = "Mustbe8characters"
          node_type          = "dc1.large"
          cluster_type       = "single-node"
          logging {
            enable = 1
          }
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from unittest import mock

import pytest
from checkov.kubernetes.image_referencer.manager import KubernetesImageReferencerManager
from checkov.common.images.image_referencer import Image
from tests.graph_utils.utils import set_graph_by_graph_framework, GRAPH_FRAMEWORKS, \
    add_vertices_to_graph_by_graph_framework


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources(graph_framework):
    # given
    resource = {
        "file_path_": "/pod.yaml",
        "__endline__": 16,
        "__startline__": 1,
        "spec": {
            "containers": [
                {
                    "name": "test-container",
                    "image": "nginx",
                },
            ],
        },
        "resource_type": "Pod",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        images = KubernetesImageReferencerManager(graph_connector=graph).extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path="/pod.yaml",
            name="nginx",
            start_line=1,
            end_line=16,
            related_resource_id="/pod.yaml:None",
        ),
    ]


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_no_duplications_while_extracting_image_names(graph_framework):
    resource = {
        "file_path_": "/pod.yaml",
        "__endline__": 16,
        "__startline__": 1,
        "spec": {
            "initContainers": [
                {
                    "name": "test-container",
                    "image": "nginx",
                },
            ],
            "containers": [
                {
                    "name": "test-container",
                    "image": "nginx",
                },
            ],
        },
        "resource_type": "Pod",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph, 1, 'duplicated_image')
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        manager = KubernetesImageReferencerManager(graph_connector=graph)
        images = manager.extract_images_from_resources()
    assert len(images) == 1
    image = images[0]
    assert image.name == 'nginx'



import copy
import os
import unittest
from pathlib import Path
from typing import Any
from unittest import mock

import pytest
from pytest_mock import MockerFixture

from checkov.common.util.consts import TRUE_AFTER_UNKNOWN
from checkov.terraform.plan_parser import parse_tf_plan, _sanitize_count_from_name, _handle_complex_after_unknown, \
    _update_after_unknown_in_complex_types
from checkov.common.parsers.node import StrNode


class TestPlanFileParser(unittest.TestCase):

    def test_tags_values_are_flattened(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_tags/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        resource_tags = resource_attributes['tags'][0]
        for tag_key, tag_value in resource_tags.items():
            if tag_key not in ['__startline__', '__endline__', '__file__', 'start_line', 'end_line']:
                self.assertIsInstance(tag_value, StrNode)

    def test_provider_is_included(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_tags/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_provider_definition = tf_definition['provider']
        self.assertTrue(file_provider_definition)  # assert a provider exists
        assert file_provider_definition[0].get('aws', {}).get('region', None) == ['us-west-2']

    def test_plan_multiple_providers(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_multiple_providers/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        providers = tf_definition['provider']
        self.assertEqual(len(providers), 3)
        provider_names = []
        provider_aliases = []
        provider_addresses = []
        for provider in providers:
            key = next(iter(provider))
            provider_names.append(key)
            provider_aliases.append(provider[key]['alias'][0])
            provider_addresses.append(provider[key]['__address__'])

        self.assertEqual(provider_names, ["aws", "aws", "aws"])
        self.assertEqual(provider_aliases, ["default", "ohio", "oregon"])
        self.assertEqual(provider_addresses, ["aws.default", "aws.ohio", "aws.oregon"])

    def test_more_tags_values_are_flattened(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_tags_variety/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        # TODO: this should also verify the flattening but at least shows it parses now.
        assert True

    # Check Plan Booleans are treated similar to normal Terraform Parser
    # https://github.com/bridgecrewio/checkov/issues/1764
    def test_simple_type_booleans_clean(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_booleans/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        self.assertTrue(resource_attributes['metadata'][0]['a'][0])
        self.assertTrue(resource_attributes['metadata'][0]['b'][0])
        self.assertFalse(resource_attributes['metadata'][0]['c'][0])
        self.assertFalse(resource_attributes['metadata'][0]['d'][0])

    def test_encodings(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        plan_files = ['tfplan_mac_utf8.json', 'tfplan_win_utf8.json', 'tfplan_win_utf16.json']

        for plan_file in plan_files:
            plan_path = os.path.join(current_dir, "resources", "plan_encodings", plan_file)
            tf_definition, _ = parse_tf_plan(plan_path, {})
            self.assertEqual(list(tf_definition['resource'][0].keys())[0], "aws_s3_bucket")

    def test_provisioners(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        plan_files = ['tfplan.json', 'tfplan2.json']

        for file in plan_files:
            valid_plan_path = current_dir + "/resources/plan_provisioners/" + file
            tf_definition, _ = parse_tf_plan(valid_plan_path, {})
            file_resource_definition = tf_definition['resource'][0]
            resource_definition = next(iter(file_resource_definition.values()))
            resource_attributes = next(iter(resource_definition.values()))
            self.assertTrue(resource_attributes['provisioner'])

    def test_module_with_connected_resources(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_module_with_connected_resources/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][1]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        self.assertTrue(resource_attributes['references_'])

    @mock.patch.dict(os.environ, {"EVAL_TF_PLAN_AFTER_UNKNOWN": "True"})
    def test_after_unknown_handling(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_after_unknown/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        self.assertEqual(resource_attributes['logging_config'][0]["bucket"], [TRUE_AFTER_UNKNOWN])

    def test___sanitize_count_from_name_with_count(self):
        name = "aws_s3_bucket.bucket[0]"
        result = _sanitize_count_from_name(name)
        self.assertEqual(result, "aws_s3_bucket.bucket")

        name = "aws_s3_bucket.bucket"
        result = _sanitize_count_from_name(name)
        self.assertEqual(result, "aws_s3_bucket.bucket")

    def test_handle_complex_after_unknown(self):
        resource = {
            "tags": [
                [
                    {
                        "custom_tags": [
                            {"key": "Tag1", "value": "Value1"},
                            {"key": "Tag2", "value": "Value2"}
                        ]
                    }
                ]
            ]
        }
        key: str = 'tags'
        value: list = [
            {
                'custom_tags': [
                    {"key": "Tag1", "value": "Value1"},
                    {"key": "Tag2", "value": "Value2"}
                ]
            }
        ]
        _handle_complex_after_unknown(key, resource, value)
        assert resource["tags"] == [value]

    def test_handle_complex_after_unknown_with_empty_list(self):
        resource = {"network_configuration": [
            {
                "endpoint_configuration": [
                ]
            }
        ]}
        key: str = 'network_configuration'
        value = [{"endpoint_configuration": []}]
        _handle_complex_after_unknown(key, resource, value)
        assert resource == {'network_configuration': [{"endpoint_configuration": []}]}

    def test_handle_complex_after_unknown_with_some_known_values(self):
        original_resource = {
            "tags": [
                {"tag1": "my_tag"},
                {"tag2": "true"},
            ]
        }
        _update_after_unknown_in_complex_types("tags", original_resource)
        assert original_resource == {
            "tags": [
                {"tag1": "my_tag"},
                {"tag2": ["true_after_unknown"]},
            ]
        }



@pytest.mark.parametrize("inner_key, k, is_inner_list", [
    ("endpoint_configuration", "network_configuration", False),
    ("endpoint_configuration", "network_configuration", True)
])
def test_handle_complex_after_unknown(inner_key: str, k: str, is_inner_list: bool) -> None:
    if is_inner_list:
        # We cannot parametrize a dict object, so we use a boolean to decide which conf to use
        resource_conf = {'network_configuration': [[{"endpoint_configuration": []}]]}
    else:
        resource_conf = {'network_configuration': [{"endpoint_configuration": []}]}
    value = [{"endpoint_configuration": []}]
    resource_conf_copy = copy.deepcopy(resource_conf)
    _handle_complex_after_unknown(k, resource_conf, value)
    assert resource_conf == resource_conf_copy


def test_large_file(mocker: MockerFixture):
    # given
    test_file = Path(__file__).parent / "resources/plan_encodings/tfplan_mac_utf8.json"

    mocker.patch("checkov.cloudformation.parser.cfn_yaml.MAX_IAC_FILE_SIZE", 1)

    # when
    tf_definition, _ = parse_tf_plan(str(test_file), {})

    assert tf_definition['resource'][0]['aws_s3_bucket']['b']['start_line'][0] == 0
    assert tf_definition['resource'][0]['aws_s3_bucket']['b']['end_line'][0] == 0

    def test_vpc_endpoint_policy_is_parsed(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_vpc_endpoint/tfplan.json"
        tf_definition, _ = parse_tf_plan(valid_plan_path, {})
        file_resource_definition = tf_definition['resource'][0]
        resource_definition = next(iter(file_resource_definition.values()))
        resource_attributes = next(iter(resource_definition.values()))
        self.assertIn('policy', resource_attributes)
        policy = resource_attributes['policy'][0]
        self.assertIn('Statement', policy)


if __name__ == '__main__':
    unittest.main()



import os
from pathlib import Path

from parameterized import parameterized_class

from checkov.runner_filter import RunnerFilter
from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class ConnectionSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(ConnectionSolver, self).setUp()

    def test_connection_found(self):
        root_folder = '../../../resources/ec2_instance_network_interfaces'
        check_id = "NetworkInterfaceForInstance"
        should_pass = ['aws_instance.instance_foo', 'aws_network_interface.network_interface_foo']
        should_fail = []
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_output_connection(self):
        root_folder = '../../../resources/output_example'
        check_id = "VPCForSubnet"
        should_pass = ['module.submodule.aws_vpc.my_vpc','aws_subnet.my_subnet']
        should_fail = []
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_data_connection(self):
        root_folder = "../../../resources/s3_bucket_policy"
        check_id = "S3BucketPolicyDataSource"
        should_pass = ["aws_s3_bucket.good"]
        should_fail = ["aws_s3_bucket.bad"]
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_reduce_graph_by_target_types(self):
        # given
        check_id = "VPCForSubnet"
        resources_path = Path(__file__).parents[3] / "resources"
        self.runner.run(root_folder=str(resources_path), runner_filter=RunnerFilter(checks=["VPCForSubnet"]))
        graph_connector = self.runner.graph_manager.db_connector.graph
        check = next(check for check in self.registry.checks if check.id == check_id)

        # when
        reduced_graph = check.solver.reduce_graph_by_target_types(graph_connector)

        # then
        if self.graph_framework == 'NETWORKX':
            assert len(graph_connector.nodes) >= 661
            assert len(graph_connector.edges) >= 327

            assert len(reduced_graph.nodes) <= 85
            assert len(reduced_graph.edges) <= 20

        elif self.graph_framework == 'RUSTWORKX':
            assert len(graph_connector.nodes()) >= 661
            assert len(graph_connector.edges()) >= 327

            assert len(reduced_graph.nodes()) <= 85
            assert len(reduced_graph.edges()) <= 20


from checkov.common.images.image_referencer import Image
from checkov.dockerfile.image_referencer.provider import DockerfileProvider


def test_extract_images_from_resources():
    # given
    definitions = {
        "/Dockerfile": {
            "FROM": [
                {
                    "instruction": "FROM",
                    "startline": 0,
                    "endline": 0,
                    "content": "FROM maven:3.8-openjdk-17-slim AS build\n",
                    "value": "maven:3.8-openjdk-17-slim AS build",
                },
                {
                    "instruction": "FROM",
                    "startline": 4,
                    "endline": 4,
                    "content": "FROM amazonlinux:2 AS run\n",
                    "value": "amazonlinux:2",
                },
            ],
            "RUN": [
                {
                    "instruction": "RUN",
                    "startline": 2,
                    "endline": 2,
                    "content": "RUN apt-get install -y curl\n",
                    "value": "apt-get install -y curl",
                },
            ],
        },
    }

    # when
    provider = DockerfileProvider(definitions=definitions)
    images = provider.extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path="/Dockerfile",
            name="amazonlinux:2",
            start_line=5,
            end_line=5,
            related_resource_id="/Dockerfile:/Dockerfile.FROM",
        ),
    ]


def test_extract_images_from_resources_with_no_image():
    # given
    definitions = {
        "/Dockerfile": {
            "RUN": [
                {
                    "instruction": "RUN",
                    "startline": 2,
                    "endline": 2,
                    "content": "RUN apt-get install -y curl\n",
                    "value": "apt-get install -y curl",
                },
            ],
        },
    }

    # when
    provider = DockerfileProvider(definitions=definitions)
    images = provider.extract_images_from_resources()

    # then
    assert not images



from __future__ import annotations

from typing import Any

import pytest

from checkov.common.bridgecrew.severities import Severity, Severities, BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.policies_3d.record import Policy3dRecord


@pytest.fixture()
def k8s_record_1() -> Record:
    return Record(
        check_name='Check 1 title', check_result={'result': CheckResult.FAILED}, file_path='/dir_a/dir_b/file1.yaml',
        file_line_range=[1, 16], resource='Pod.default.testPod1', check_class='', bc_check_id='BC_K8S_1',
        code_block=[(1, 'apiVersion: v1\n'), (2, 'kind: Pod\n'), (3, 'metadata:\n'),
                    (4, '  name: 3d-policy-3\n'), (5, 'spec:\n'), (6, '  containers:\n'),
                    (7, '    - name: demo 1\n'), (8, '      image: ubuntu:latest\n'),
                    (9, '      securityContext:\n'), (10, '        runAsNonRoot: false\n'),
                    (11, '    - name: demo 2\n'), (12, '      image: ubuntu:latest\n'),
                    (13, '      securityContext:\n'), (14, '        runAsUser: 0\n'),
                    (15, '    - name: demo 3\n'), (16, '      image: ubuntu:latest\n')],
        file_abs_path='/root/dir_a/dir_b/file1.yaml', evaluations={}, check_id='', severity=Severities[BcSeverities.MEDIUM])


@pytest.fixture()
def k8s_record_2() -> Record:
    return Record(
        check_name='Check 2 title', check_result={'result': CheckResult.FAILED}, file_path='/dir_a/dir_b/file1.yaml',
        file_line_range=[1, 16], resource='Pod.default.testPod1', check_class='', bc_check_id='BC_K8S_2',
        code_block=[(1, 'apiVersion: v1\n'), (2, 'kind: Pod\n'), (3, 'metadata:\n'),
                    (4, '  name: 3d-policy-3\n'), (5, 'spec:\n'), (6, '  containers:\n'),
                    (7, '    - name: demo 1\n'), (8, '      image: ubuntu:latest\n'),
                    (9, '      securityContext:\n'), (10, '        runAsNonRoot: false\n'),
                    (11, '    - name: demo 2\n'), (12, '      image: ubuntu:latest\n'),
                    (13, '      securityContext:\n'), (14, '        runAsUser: 0\n'),
                    (15, '    - name: demo 3\n'), (16, '      image: ubuntu:latest\n')],
        file_abs_path='/root/dir_a/dir_b/file1.yaml', evaluations={}, check_id='', severity=Severities[BcSeverities.LOW])


@pytest.fixture()
def k8s_record_3() -> Record:
    return Record(
        check_name='Check 3 title', check_result={'result': CheckResult.FAILED}, file_path='/dir_a/dir_b/file2.yaml',
        file_line_range=[1, 16], resource='Pod.default.testPod2', check_class='', bc_check_id='BC_K8S_3',
        code_block=[(1, 'apiVersion: v1\n'), (2, 'kind: Pod\n'), (3, 'metadata:\n'),
                    (4, '  name: 3d-policy-3\n'), (5, 'spec:\n'), (6, '  containers:\n'),
                    (7, '    - name: demo 1\n'), (8, '      image: image-with-no-cves\n'),
                    (9, '      securityContext:\n'), (10, '        runAsNonRoot: false\n'),
                    (11, '    - name: demo 2\n'), (12, '      image: ubuntu:latest\n'),
                    (13, '      securityContext:\n'), (14, '        runAsUser: 0\n'),
                    (15, '    - name: demo 3\n'), (16, '      image: ubuntu:latest\n')],
        file_abs_path='/root/dir_a/dir_b/file2.yaml', evaluations={}, check_id='', severity=Severities[BcSeverities.HIGH])


@pytest.fixture()
def k8s_report(k8s_record_1, k8s_record_2, k8s_record_3) -> Report:
    report = Report(check_type='kubernetes')
    report.add_record(k8s_record_1)
    report.add_record(k8s_record_2)
    report.add_record(k8s_record_3)
    return report


@pytest.fixture()
def k8s_report_2(k8s_record_1, k8s_record_3) -> Report:
    report = Report(check_type='kubernetes')
    report.add_record(k8s_record_1)
    report.add_record(k8s_record_3)
    return report


@pytest.fixture()
def cve_1() -> dict[str, Any]:
    return {'cveId': 'CVE-2022-42898', 'status': 'fixed in 1.19.2-2ubuntu0.1', 'severity': 'medium',
                  'packageName': 'krb5', 'packageVersion': '1.19.2-2', 'dockerImageName': 'ubuntu:latest',
                  'link': 'https://people.canonical.com/~ubuntu-security/cve/2022/CVE-2022-42898',
                  'cvss': 8.8, 'vector': 'CVSS:3.1/AV:N/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H',
                  'description': 'PAC parsing in MIT Kerberos 5 (aka krb5) before 1.19.4 ...',
                  'riskFactors': ['DoS', 'Has fix', 'Medium severity', 'Recent vulnerability',
                                  'Remote execution', 'Attack complexity: low', 'Attack vector: network'],
                  'publishedDate': '2022-12-25T06:15:00Z'}

@pytest.fixture()
def cve_2() -> dict[str, Any]:
    return {'cveId': 'CVE-2022-3821', 'status': 'needed', 'severity': 'medium',
              'packageName': 'systemd', 'packageVersion': '249.11-0ubuntu3.6', 'dockerImageName': 'ubuntu:latest',
              'link': 'https://people.canonical.com/~ubuntu-security/cve/2022/CVE-2022-3821',
              'cvss': 5.5, 'vector': 'CVSS:3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:N/I:N/A:H',
              'description': 'An off-by-one Error issue was discovered in Systemd in ...',
              'riskFactors': ['DoS', 'Medium severity', 'Recent vulnerability',
                              'Attack complexity: low'], 'publishedDate': '2022-11-08T22:15:00Z'}

@pytest.fixture()
def sca_image_cached_results(cve_1, cve_2) -> list[dict[str, Any]]:
    return [{'dockerImageName': 'ubuntu:latest', 'dockerFilePath': '/dir_a/dir_b/file1.yaml',
             'dockerFileContent': 'image: ubuntu:latest', 'type': 'Image',
             'sourceId': 'owner-name/repo-name', 'branch': 'branch-name', 'sourceType': 'cli',
             'relatedResourceId': '/root/dir_a/dir_b/file1.yaml:Pod.default.testPod1',
             'vulnerabilities': [cve_1, cve_2]}]


@pytest.fixture()
def sca_image_report(sca_image_cached_results) -> Report:
    report = Report(check_type='sca_image')
    report.image_cached_results = sca_image_cached_results
    return report


@pytest.fixture()
def scan_reports(k8s_report, sca_image_report) -> list[Report]:
    return [k8s_report, sca_image_report]

@pytest.fixture()
def scan_reports_2(k8s_report_2, sca_image_report) -> list[Report]:
    return [k8s_report_2, sca_image_report]


@pytest.fixture()
def policy_3d_1() -> dict[str, Any]:
    return {'id': 'CKV_P3D_1', 'title': '3d policy 1', 'guideline': 'guideline-1', 'severity': 'CRITICAL',
               'category': 'Policy3D', 'code': '{"iac":{"kubernetes":["BC_K8S_1"]},"cve":{"risk_factor":["DoS"]}}'}

@pytest.fixture()
def policy_3d_2() -> dict[str, Any]:
    return {'id': 'CKV_P3D_2', 'title': '3d policy 2', 'guideline': 'guideline-2', 'severity': 'CRITICAL',
               'category': 'Policy3D',
               'code': '{"iac":{"kubernetes":["BC_K8S_2"]},"cve":{"risk_factor":["Recent vulnerability"]}}'}

@pytest.fixture()
def policy_3d_3() -> dict[str, Any]:
    return {'id': 'CKV_P3D_3', 'title': '3d policy 3', 'guideline': 'guideline-3', 'severity': 'CRITICAL',
               'category': 'Policy3D',
               'code': '{"iac":{"kubernetes":["BC_K8S_1", "BC_K8S_2"]},"cve":{"risk_factor":["Recent vulnerability"]}}'}

@pytest.fixture
def raw_3d_policy():
  return {'id': 'BC_3D_500', 'title': 'title_500', 'guideline': 'guideline_500',
          'severity': 'CRITICAL', 'pcSeverity': 'CRITICAL', 'category': 'Policy3D',
          'code': """{
            "version": "v1",
            "definition": [
              {
                "cves": {
                  "or": [
                    {
                      "and": [
                        {
                          "risk_factor": "DoS"
                        },
                        {
                          "risk_factor": "Medium Severity"
                        }
                      ]
                    }
                  ]
                }
              },
              {
                "iac": {
                  "or": [
                    {
                      "violation_id": "BC_K8S_1"
                    },
                    {
                      "violation_id": "BC_K8S_23"
                    }
                  ]
                }
              }
            ]
          }"""
        }

@pytest.fixture()
def policy_3d_record_single_iac_single_cve(k8s_record_1, cve_1) -> Policy3dRecord:
    record = Policy3dRecord(
        bc_check_id='BC_P3D_1',
        check_id='BC_P3D_1',
        check_name='3d policy 1',
        check_result={'result': CheckResult.FAILED},
        code_block=[],
        evaluations=None,
        file_path='',
        file_abs_path='',
        resource='',
        check_class='',
        file_line_range=[-1, -1],
        iac_records=[k8s_record_1],
        vulnerabilities=[cve_1],
        severity=Severities[BcSeverities.LOW],
        composed_from_iac_records=[],
        composed_from_secrets_records=[],
        composed_from_cves=[]
    )
    record.set_guideline('https://docs.bridgecrew.io/docs/bc_p3d_1')
    return record

@pytest.fixture()
def policy_3d_record_multi_iac_multi_cve(k8s_record_1, k8s_record_2, k8s_record_3, cve_1, cve_2) -> Policy3dRecord:
    record = Policy3dRecord(
        bc_check_id='BC_P3D_1',
        check_id='BC_P3D_1',
        check_name='3d policy 1',
        check_result={'result': CheckResult.FAILED},
        code_block=[],
        evaluations=None,
        file_path='',
        file_abs_path='',
        resource='',
        check_class='',
        file_line_range=[-1, -1],
        iac_records=[k8s_record_1, k8s_record_2, k8s_record_3],
        vulnerabilities=[cve_1, cve_2],
        severity=Severities[BcSeverities.LOW],
        composed_from_iac_records=[],
        composed_from_secrets_records=[],
        composed_from_cves=[]
    )
    record.set_guideline('https://docs.bridgecrew.io/docs/bc_p3d_1')
    return record


from pathlib import Path
from typing import Dict

from checkov.common.util.parser_utils import eval_string
from checkov.terraform.tf_parser import load_or_die_quietly


def test_eval_string_to_list() -> None:
    # given
    expected = ["a", "b", "c"]

    # when
    actual = eval_string('["a", "b", "c"]')

    assert actual == expected


def test__load_or_die_quietly_with_bom() -> None:
    # given
    test_file = Path(__file__).parent / "resources/file_bom/with_bom.tf"
    parsing_errors: Dict[str, Exception] = {}

    # when
    definition = load_or_die_quietly(file=test_file, parsing_errors=parsing_errors)

    # then
    assert not parsing_errors
    assert definition == {
        "resource": [
            {
                "aws_s3_bucket": {
                    "example": {"bucket": ["example"], "__start_line__": 1, "__end_line__": 3},
                },
            }
        ]
    }


def test__load_or_die_quietly_without_bom() -> None:
    # given
    test_file = Path(__file__).parent / "resources/file_bom/without_bom.tf"
    parsing_errors: Dict[str, Exception] = {}

    # when
    definition = load_or_die_quietly(file=test_file, parsing_errors=parsing_errors)

    # then
    assert not parsing_errors
    assert definition == {
        "resource": [
            {
                "aws_s3_bucket": {
                    "example": {"bucket": ["example"], "__start_line__": 1, "__end_line__": 3},
                },
            }
        ]
    }


def test__load_or_die_quietly_with_timeout() -> None:
    test_file = Path(__file__).parent / "resources/hcl_timeout/main.tf"
    parsing_errors: Dict[str, Exception] = {}

    # when
    definition = load_or_die_quietly(file=test_file, parsing_errors=parsing_errors)

    # then
    assert parsing_errors
    assert str(test_file) in parsing_errors
    assert 'seconds to parse' in str(parsing_errors[str(test_file)])


import os
import unittest

from checkov.terraform.plan_runner import Runner
from checkov.runner_filter import RunnerFilter


class TestTFplanSkipCheckRegex(unittest.TestCase):

    def test_skip_all_checks(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/resource"
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=[
                "CKV_AWS_*:.*.json$",
                "CKV2_AWS_*:.*.json$",
                "CKV_NCP_*:.*.json$",
                "CKV_AZURE_*:.*.json$",
                "CKV2_AZURE_*:.*.json$",
                "CKV2_GCP_*:.*.json$",
                "CKV_ADO_*:.*.json$",
                "CKV2_ADO_*:.*.json$",
                "CKV_OCI_*:.*.json$",
                "CKV2_OCI_*:.*.json$",
                "CKV_GIT_*:.*.json$",
                "CKV2_GIT_1:.*.json$"
            ])
        )
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 0)
        # As skip is not being inserted to result in base check scan
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_some_checks(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/resource"
        report1 = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV2_AWS_*:.*.json$"])
        )
        summary1 = report1.get_summary()

        report2 = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter())
        summary2 = report2.get_summary()

        self.assertNotEqual(summary1['passed'], summary2['passed'])
        self.assertNotEqual(summary1['failed'], summary2['failed'])

    def test_skip_only_one_file(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/resource"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(skip_checks=[
                "CKV_AWS_*:.*skip_directory.*.json$",
                "CKV2_AWS_*:.*skip_directory.*.json$",
                "CKV_NCP_*:.*skip_directory.*.json$",
                "CKV_AZURE_*:.*skip_directory.*.json$",
                "CKV2_AZURE_*:.*skip_directory.*.json$",
                "CKV2_GCP_*:.*skip_directory.*.json$",
                "CKV_ADO_*:.*skip_directory.*.json$",
                "CKV2_ADO_*:.*skip_directory.*.json$",
                "CKV_OCI_*:.*skip_directory.*.json$",
                "CKV2_OCI_*:.*skip_directory.*.json$",
                "CKV_GIT_*:.*skip_directory.*.json$",
                "CKV2_GIT_1:.*skip_directory.*.json$"
            ]))

        summary = report.get_summary()
        no_skip_report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter())
        no_skip_summary = no_skip_report.get_summary()
        self.assertNotEqual(summary['passed'], no_skip_summary['passed'])
        self.assertNotEqual(summary['failed'], no_skip_summary['failed'])


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.GlobalAcceleratorAcceleratorFlowLogs import check


class TestGlobalAcceleratorAcceleratorFlowLogs(unittest.TestCase):

    def test_failure(self):
        resource_conf = {"name": "Example"}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_full(self):
        resource_conf = {
            "name": "Example",
            "attributes": [
                {
                    "flow_logs_enabled": [False]
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "name": "Example",
            "attributes": [
                {
                    "flow_logs_enabled": [True]
                }
            ],
        }

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pprint
import unittest

from typing import List, Tuple

from checkov.common.util.parser_utils import VarBlockMatch as VBM, split_merge_args, find_var_blocks


class TestParserInternals(unittest.TestCase):
    def test_split_merge_args(self):
        cases: List[Tuple[str, List[str]]] = [
            ("local.one, local.two",
             ["local.one", "local.two"]),
            ("{Tag4 = \"four\"}, {Tag5 = \"five\"}",
             ["{Tag4 = \"four\"}", "{Tag5 = \"five\"}"]),
            ("{a=\"b\"}, {a=[1,2], c=\"z\"}, {d=3}",
             ["{a=\"b\"}", "{a=[1,2], c=\"z\"}", "{d=3}"]),
            ("local.common_tags, merge({Tag4 = \"four\"}, {Tag5 = \"five\"})",
             ["local.common_tags", "merge({Tag4 = \"four\"}, {Tag5 = \"five\"})"]),
            (", ",
             None),
            ("",
             None),
            (", leading_comma",
             ["leading_comma"]),
            ("kinda_maybe_shouldnt_work_but_we_will_roll_with_it, ",        # <-- trailing comma
             ["kinda_maybe_shouldnt_work_but_we_will_roll_with_it"]),
            ("local.one",
             ["local.one"]),
            ('{"a": "}, evil"}',        # bracket inside string, should not be split
             ['{"a": "}, evil"}']),
            ("{'a': '}, evil'}",        # bracket inside string, should not be split
             ["{'a': '}, evil'}"]),     # Note: these happen with native maps (see merge tests)
            ('${merge({\'a\': \'}, evil\'})}',
             ['${merge({\'a\': \'}, evil\'})}']),
            ('local.common_tags,,{\'Tag4\': \'four\'},,{\'Tag2\': \'Dev\'},',
             ["local.common_tags", "{\'Tag4\': \'four\'}", "{\'Tag2\': \'Dev\'}"])
        ]
        for case in cases:
            actual = split_merge_args(case[0])
            assert actual == case[1], f"Case \"{case[0]}\" failed. Expected: {case[1]}  Actual: {actual}"

    def test_find_var_blocks(self):
        cases: List[Tuple[str, List[VBM]]] = [
            (
                "${local.one}",
                [
                    VBM("${local.one}", "local.one")
                ]
            ),
            (
                "${merge({a=\"b\"}, {a=[1,2], c=\"z\"}, {d=3})}",
                [
                    VBM("${merge({a=\"b\"}, {a=[1,2], c=\"z\"}, {d=3})}",
                        "merge({a=\"b\"}, {a=[1,2], c=\"z\"}, {d=3})")
                ]
            ),
            (
                "\"string$ ${tomap({key=\"value\"})[key]} are fun\"",
                [
                    VBM("${tomap({key=\"value\"})[key]}", "tomap({key=\"value\"})[key]")
                ]
            ),
            # This case highlights that inner evals should be returned
            (
                "${filemd5(\"${path.module}/templates/some-file.json\")}",
                [
                    VBM("${path.module}", "path.module"),
                    VBM("${filemd5(\"${path.module}/templates/some-file.json\")}",
                        "filemd5(\"${path.module}/templates/some-file.json\")")
                ]
            ),
            (
                "${local.NAME[foo]}-${local.TAIL}${var.gratuitous_var_default}",
                [
                    VBM("${local.NAME[foo]}", "local.NAME[foo]"),
                    VBM("${local.TAIL}", "local.TAIL"),
                    VBM("${var.gratuitous_var_default}", "var.gratuitous_var_default")
                ]
            ),
            (
                "${tostring(\"annoying {\")}",
                [
                    VBM("${tostring(\"annoying {\")}", "tostring(\"annoying {\")")
                ]
            ),
            (
                "${tostring(\"annoying }\")}",
                [
                    VBM("${tostring(\"annoying }\")}", "tostring(\"annoying }\")")
                ]
            ),
            (
                "${this-is-unterminated",
                []
            ),
            (
                "${merge({\"a\": \"}, evil\"},{\"b\": \"\\\" , evil\"})}",
                [
                    VBM("${merge({\"a\": \"}, evil\"},{\"b\": \"\\\" , evil\"})}",
                        "merge({\"a\": \"}, evil\"},{\"b\": \"\\\" , evil\"})")
                ]
            ),
            (
                "$${foo}",          # escape interpolation
                []
            ),
            (
                '${merge({\'a\': \'}, evil\'})}',
                [
                    VBM('${merge({\'a\': \'}, evil\'})}', 'merge({\'a\': \'}, evil\'})')
                ]
            ),

            # Ordered returning of sub-vars and then outer var.
            (
                "${merge(local.common_tags,local.common_data_tags,{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                [
                    VBM("local.common_tags", "local.common_tags"),
                    VBM("local.common_data_tags", "local.common_data_tags"),
                    VBM("${var.ENVIRONMENT}", "var.ENVIRONMENT"),
                    VBM("${var.REGION}", "var.REGION"),
                    VBM("${merge(local.common_tags,local.common_data_tags,{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                        "merge(local.common_tags,local.common_data_tags,{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})")
                ]
            ),
            (
                "${merge(${local.common_tags},${local.common_data_tags},{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                [
                    VBM("${local.common_tags}", "local.common_tags"),
                    VBM("${local.common_data_tags}", "local.common_data_tags"),
                    VBM("${var.ENVIRONMENT}", "var.ENVIRONMENT"),
                    VBM("${var.REGION}", "var.REGION"),
                    VBM("${merge(${local.common_tags},${local.common_data_tags},{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                        "merge(${local.common_tags},${local.common_data_tags},{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})")
                ]
            ),
            (
                '${merge(var.tags, map("Name", "${var.name}", "data_classification", "none"))}',
                [
                    VBM("var.tags", "var.tags"),
                    VBM("${var.name}", "var.name"),
                    VBM('map("Name", "${var.name}", "data_classification", "none")',
                        'map("Name", "${var.name}", "data_classification", "none")'),
                    VBM('${merge(var.tags, map("Name", "${var.name}", "data_classification", "none"))}',
                        'merge(var.tags, map("Name", "${var.name}", "data_classification", "none"))')
                ]
            ),

            # Ternaries
            (
                '${var.metadata_http_tokens_required ? "required" : "optional"}',
                [
                    VBM('var.metadata_http_tokens_required', 'var.metadata_http_tokens_required'),
                    VBM('${var.metadata_http_tokens_required ? "required" : "optional"}',
                        'var.metadata_http_tokens_required ? "required" : "optional"')
                ]
            ),
            (
                '${1 + 1 == 2 ? "required" : "optional"}',
                [
                    VBM('1 + 1 == 2', '1 + 1 == 2'),
                    VBM('${1 + 1 == 2 ? "required" : "optional"}', '1 + 1 == 2 ? "required" : "optional"')
                ]
            ),
            (
                '${true ? "required" : "optional"}',
                [
                    VBM('${true ? "required" : "optional"}', 'true ? "required" : "optional"')
                ]
            ),
            (
                '${false ? "required" : "optional"}',
                [
                    VBM('${false ? "required" : "optional"}', 'false ? "required" : "optional"')
                ]
            ),
            # TODO: var -> comparison -> ternary
            # (
            #     '${local.empty != "" ? local.a : "default value"}',
            #     [
            #         VBM("local.empty", "local.empty"),
            #         VBM('local.empty != ""', 'local.empty != ""'),
            #         VBM('${local.empty != "" ? local.a : "default value"}',
            #             'local.empty != "" ? local.a : "default value"')
            #     ]
            # )
        ]
        for case in cases:
            actual = find_var_blocks(case[0])
            assert actual == case[1], \
                f"Case \"{case[0]}\" failed ❌:\n" \
                f"  Expected: \n{pprint.pformat(case[1], indent=2)}\n\n" \
                f"  Actual: \n{pprint.pformat(actual, indent=2)}"
            print(f"Case \"{case[0]}: ✅")



from aws_cdk import core
from aws_cdk import aws_lambda as _lambda
from aws_cdk import aws_sam as sam
class MyLambdaFunctionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Lambda function
        my_lambda_function = _lambda.Function(
            self, 'MyLambdaFunction',
            runtime=_lambda.Runtime.PYTHON_3_8,
            handler='index.handler',
            code=_lambda.Code.from_asset('path/to/your/function/code'),
            environment={
                'MY_VARIABLE_1': 'Value1',
                'MY_VARIABLE_2': 'Value2'
            },
            kms_key=_lambda.Key.from_key_arn(self, 'MyKmsKey', 'arn:aws:kms:region:account-id:key/key-id')
        )

app = core.App()
MyLambdaFunctionStack(app, "MyLambdaFunctionStack")
app.synth()


class MyServerlessFunctionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define SAM Function
        my_sam_function = sam.CfnFunction(
            self, 'MySAMFunction',
            handler='index.handler',
            runtime='python3.9',
            code_uri='./path/to/your/function/code',
            environment={
                'MY_VARIABLE_1': 'Value1',
                'MY_VARIABLE_2': 'Value2'
            },
            kms_key_arn='arn:aws:kms:region:account-id:key/key-id'
        )

app = core.App()
MyServerlessFunctionStack(app, "MyServerlessFunctionStack")
app.synth()



from aws_cdk import core
from aws_cdk.aws_lambda import Function, Runtime, Code
from aws_cdk.aws_sam import CfnFunction

class MyLambdaStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        my_lambda_function = Function(
            self,
            "MyLambdaFunction",
            runtime=Runtime.PYTHON_3_8,  # Set the Lambda function's runtime
            handler="index.handler",  # Specify the Lambda handler
            code=Code.from_asset("path/to/your/code"),  # Define the code location
            function_name="my-function-name",  # Optionally set the function name
            role=my_lambda_execution_role,  # Provide an IAM role for the function
            timeout=core.Duration.seconds(10),  # Set the function timeout
            reserved_concurrent_executions=5
        )
class MyLambdaStack2(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        my_lambda_function = CfnFunction(
            self,
            "MyLambdaFunction",
            handler="index.handler",  # Specify the Lambda handler
            runtime="nodejs14.x",  # Set the Lambda function's runtime
            code_uri="./my-code",  # Specify the location of your code
            function_name="my-function-name",  # Optionally set the function name
            role=my_lambda_execution_role,  # Provide an IAM role for the function
            timeout=10,  # Set the function timeout
            reserved_concurrent_executions=5
        )

        # You can add other configurations and permissions for your Lambda function here





from copy import copy, deepcopy
from checkov.common.checks.base_check_registry import BaseCheckRegistry
import pytest

@pytest.fixture(scope='module', autouse=True)
def clean_bc_integration() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.clean()


@pytest.fixture(scope='module', autouse=True)
def clean_feature_registry():
    from checkov.common.bridgecrew.integration_features.integration_feature_registry import integration_feature_registry
    old_features = copy(integration_feature_registry.features)
    before_registered_checks = copy(BaseCheckRegistry._BaseCheckRegistry__all_registered_checks)
    yield
    integration_feature_registry.features = old_features
    BaseCheckRegistry._BaseCheckRegistry__all_registered_checks = before_registered_checks




@pytest.fixture(scope='module', autouse=True)
def reset_checks():
    from checkov.terraform.checks.resource.registry import resource_registry as registry
    before_checks = deepcopy(registry.checks)
    before_wildcards_checks = deepcopy(registry.wildcard_checks)
    yield
    registry.checks = deepcopy(before_checks)
    registry.wildcard_checks = deepcopy(before_wildcards_checks)


import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.terraform.checks.resource.registry import resource_registry

class TestDynamicCheck(BaseResourceValueCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Ensure it ain't broke"
        id = "test/TestDynamicCheck"
        categories = []
        supported_resources = ["doesnt_matter"]
        guideline = "https://docs.prismacloud.io/policy-reference/test-policies/test-dynamic-check"
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            guideline=guideline
        )

    def get_inspected_key(self):
        return "dynamic_block_name/[0]/foo"

    def get_expected_value(self):
        return "bar"

class TestNestedDynamicCheck(BaseResourceValueCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Ensure it ain't broke"
        id = "test/TestNestedDynamicCheck"
        categories = []
        supported_resources = ["doesnt_matter"]
        guideline = "https://docs.prismacloud.io/policy-reference/test-policies/test-nested-dynamic-check"
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            guideline=guideline
        )
        
    def get_inspected_key(self):
        return "outside/dynamic_block_name/[0]/foo"

    def get_expected_value(self):
        return "bar"


class TestNestedMultipleDynamicCheckBlock1(BaseResourceValueCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Ensure it ain't broke"
        id = "test/TestNestedMultipleDynamicCheckBlock1"
        categories = []
        supported_resources = ["doesnt_matter"]
        guideline = "https://docs.prismacloud.io/policy-reference/test-policies/test-nested-multiple-dynamic-check-block-1"
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            guideline=guideline
        )

    def get_inspected_key(self):
        return "outside/dynamic_block_name/[0]/dynamic_block_1/[0]/key"

    def get_expected_value(self):
        return 1

class TestNestedMultipleDynamicCheckBlock2(BaseResourceValueCheck):
    # for pytest not to collect this class as tests
    __test__ = False

    def __init__(self):
        name = "Ensure it ain't broke"
        id = "test/TestNestedMultipleDynamicCheckBlock2"
        categories = []
        supported_resources = ["doesnt_matter"]
        guideline = "https://docs.prismacloud.io/policy-reference/test-policies/test-nested-multiple-dynamic-check-block-2"
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            guideline=guideline
        )

    def get_inspected_key(self):
        return "outside/dynamic_block_name/[0]/dynamic_block_2/[0]/key"

    def get_expected_value(self):
        return "2"

class Test(unittest.TestCase):
    def test_dynamic(self):
        data = {
            "dynamic": [{
                "dynamic_block_name": {
                    "content": {
                        "foo": "bar"
                    }
                }
            }]
        }

        result = self._check(TestDynamicCheck(),data)
        self.assertEqual(result, CheckResult.PASSED)

    def test_dynamic_nested(self):
        data = {
            "outside": {
                "dynamic": [{
                    "dynamic_block_name": {
                        "content": {
                            "foo": "bar"
                        }
                    }
                }]
            }
        }

        result = self._check(TestNestedDynamicCheck(), data)
        self.assertEqual(result, CheckResult.PASSED)

    def multipleDynamicBlockData(self):
        return {
            "outside": {
                "dynamic": [{
                    "dynamic_block_name": {
                        "content": {
                            "dynamic": [{
                                "dynamic_block_1": {
                                    "content": {
                                        "key": 1
                                    }
                                },
                                "dynamic_block_2": {
                                    "content": {
                                        "key": "2"
                                    }
                                }
                            }]
                        }
                    }
                }]
            }
        }

    def test_nested_multiple_dynamic_block_1(self):
        result = self._check(TestNestedMultipleDynamicCheckBlock1(), self.multipleDynamicBlockData())
        self.assertEqual(result, CheckResult.PASSED)

    def test_nested_multiple_dynamic_block_2(self):
        result = self._check(TestNestedMultipleDynamicCheckBlock2(), self.multipleDynamicBlockData())
        self.assertEqual(result, CheckResult.PASSED)


    @staticmethod
    def _check(check, data):
        return check.scan_resource_conf(data)

    # This will install a custom check, so setUp/tearDown will ensure the check list is unchanged
    # globally by our changes.
    def setUp(self) -> None:
        self.check_list_before = resource_registry.checks.copy()  # copy
        super().setUp()

    def tearDown(self) -> None:
        super().tearDown()
        resource_registry.checks = self.check_list_before
        self.check_list_before = None





import os
import unittest

from checkov.kubernetes.checks.resource.k8s.PeerClientCertAuthTrue import check
from checkov.kubernetes.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestPeerClientCertAuthTrue(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_PeerClientCertAuthTrue"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()
        self.assertEqual(1, summary['passed'])
        self.assertEqual(2, summary['failed'])
        for failed in report.failed_checks:
            self.assertIn("should-fail", failed.resource)
        for passed in report.passed_checks:
            self.assertIn("should-pass", passed.resource)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
from typing import Any

import pytest
from checkov.common.secrets.consts import ValidationStatus

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult

from checkov.common.output.secrets_record import SecretsRecord

from checkov.common.output.report import Report


@pytest.fixture
def secrets_report() -> Report:
    kwargs = {'check_id': 'mock', 'check_name': 'mock', 'code_block': 'mock', 'file_path': 'mock',
              'file_line_range': 'mock', 'evaluations': 'mock', 'check_class': 'mock', 'file_abs_path': 'mock'}
    record_1 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1",
                             check_result={"result": CheckResult.FAILED},
                             validation_status=ValidationStatus.VALID.value, **kwargs)
    record_2 = SecretsRecord(bc_check_id="VIOLATION_2", resource="RESOURCE_2",
                             check_result={"result": CheckResult.FAILED},
                             validation_status=ValidationStatus.INVALID.value, **kwargs)
    record_3 = SecretsRecord(bc_check_id="VIOLATION_3", resource="RESOURCE_3",
                             check_result={"result": CheckResult.FAILED},
                             validation_status=ValidationStatus.UNKNOWN.value, **kwargs)
    record_4 = SecretsRecord(bc_check_id="VIOLATION_4", resource="RESOURCE_4",
                             check_result={"result": CheckResult.FAILED},
                             validation_status=ValidationStatus.VALID.value, **kwargs)

    record_5 = SecretsRecord(bc_check_id="VIOLATION_1", resource="RESOURCE_1",
                             check_result={"result": CheckResult.PASSED},
                             validation_status=ValidationStatus.INVALID.value, **kwargs)

    report = Report(CheckType.SECRETS)
    report.add_record(record_1)
    report.add_record(record_2)
    report.add_record(record_3)
    report.add_record(record_4)
    report.add_record(record_5)

    return report


@pytest.fixture()
def json_reduced_check() -> dict[str, Any]:
    return {
        "check_id": "CKV_GHA_1",
        "check_name": "Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables",
        "check_result": {
            "result": "PASSED",
            "results_configuration": {}
        },
        "resource": "jobs(container-test-job)",
        "file_path": "/.github/workflows/image_no_violation.yml",
        "file_line_range": [
            7,
            7
        ],
        "file_abs_path": "/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml",
        "code_block": [
            [
                7,
                "    runs-on: ubuntu-latest\n"
            ],
        ],
        "bc_check_id": "BC_REPO_GITHUB_ACTION_1",
        "inspected_key_line": None,
        "evaluated_keys": None,
        "inspected_key": "",
        "inspected_value": ""
    }

@pytest.fixture()
def json_reduced_report() -> dict[str, Any]:
    return {
        "checks": {
            "passed_checks": [
                {
                    "check_id": "CKV_GHA_1",
                    "check_name": "Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables",
                    "check_result": {
                        "result": "PASSED",
                        "results_configuration": {}
                    },
                    "resource": "jobs(container-test-job)",
                    "file_path": "/.github/workflows/image_no_violation.yml",
                    "file_line_range": [
                        7,
                        7
                    ],
                    "file_abs_path": "/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml",
                    "code_block": [
                        [
                            7,
                            "    runs-on: ubuntu-latest\n"
                        ],
                    ],
                    "bc_check_id": "BC_REPO_GITHUB_ACTION_1",
                    "inspected_key_line": None,
                    "evaluated_keys": None,
                    "inspected_key": "",
                    "inspected_value": ""
                }
            ],
            "failed_checks": [
                {
                    "check_id": "CKV_GHA_2",
                    "check_name": "Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables",
                    "check_result": {
                        "result": "FAILED",
                        "results_configuration": {}
                    },
                    "resource": "jobs(container-test-job)",
                    "file_path": "/.github/workflows/image_no_violation.yml",
                    "file_line_range": [
                        7,
                        7
                    ],
                    "file_abs_path": "/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml",
                    "code_block": [
                        [
                            7,
                            "    runs-on: ubuntu-latest\n"
                        ],
                    ],
                    "bc_check_id": "BC_REPO_GITHUB_ACTION_1",
                    "inspected_key_line": None,
                    "evaluated_keys": None,
                    "inspected_key": "",
                    "inspected_value": ""
                }
            ],
            "skipped_checks": []
        },
        "image_cached_results": []
    }


from pathlib import Path
from unittest import mock
from unittest.mock import MagicMock

import click
import pytest
from _pytest.capture import CaptureFixture
from checkov.common.util import prompt


@pytest.fixture(autouse=True)
def checkov_root_mock(tmp_path: Path):
    with mock.patch("checkov.common.util.prompt.CHECKOV_ROOT_DIRECTORY", str(tmp_path / "checkov")):
        yield


def test_prompt_terraform_aws_resource(capsys: CaptureFixture[str]):
    test_name = "AWSTestPromptUnitTest"
    choices = [
        "add",
        f"{test_name}",
        "iam",
        "Tests for the AWS Prompt Unit Test",
        "terraform",
        "aws",
        "resource",
        "aws_iam_policy",
    ]

    mock_click = click
    mock_click.prompt = MagicMock(name="prompt", side_effect=choices)
    resp = prompt.Prompt()
    check = prompt.Check(resp.responses)
    check.action()

    captured = capsys.readouterr()

    expected = [
        f"Creating Check {test_name}.py",
        f"Creating Unit Test Stubs for {test_name}",
        "Successfully created",
        f"checkov/terraform/checks/resource/aws/{test_name}.py",
        "Next steps:",
    ]

    for exp in expected:
        assert exp in captured.out


def test_prompt_terraform_azure_resource(capsys: CaptureFixture[str]):
    test_name = "AzureTestPromptUnitTest"
    choices = [
        "add",
        f"{test_name}",
        "iam",
        "Tests for the Azure Prompt Unit Test",
        "terraform",
        "azure",
        "resource",
        "azurerm_policy_definition",
    ]

    mock_click = click
    mock_click.prompt = MagicMock(name="prompt", side_effect=choices)
    resp = prompt.Prompt()
    check = prompt.Check(resp.responses)
    check.action()

    captured = capsys.readouterr()

    expected = [
        f"Creating Check {test_name}.py",
        f"Creating Unit Test Stubs for {test_name}",
        "Successfully created",
        f"checkov/terraform/checks/resource/azure/{test_name}.py",
        "Next steps:",
    ]

    for exp in expected:
        assert exp in captured.out


def test_prompt_terraform_gcp_resource(capsys: CaptureFixture[str]):
    test_name = "GCPTestPromptUnitTest"
    choices = [
        "add",
        f"{test_name}",
        "iam",
        "Tests for the GCP Prompt Unit Test",
        "terraform",
        "gcp",
        "resource",
        "google_project_iam_policy",
    ]

    mock_click = click
    mock_click.prompt = MagicMock(name="prompt", side_effect=choices)
    resp = prompt.Prompt()
    check = prompt.Check(resp.responses)
    check.action()

    captured = capsys.readouterr()

    expected = [
        f"Creating Check {test_name}.py",
        f"Creating Unit Test Stubs for {test_name}",
        "Successfully created",
        f"checkov/terraform/checks/resource/gcp/{test_name}.py",
        "Next steps:",
    ]

    for exp in expected:
        assert exp in captured.out



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestRangeIncludesSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestRangeIncludesSolver, self).setUp()

    def test_range_includes_int_solver(self):
        root_folder = 'resources'
        check_id = "RangeIncludesInt"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_string_solver(self):
        root_folder = 'resources'
        check_id = "RangeIncludesString"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_int_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeIncludesInt"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_string_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeIncludesString"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_list_solver(self):
        root_folder = 'resources'
        check_id = "RangeIncludesList"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeIncludesList"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_list_w_list_solver(self):
        root_folder = 'resources'
        check_id = "RangeIncludesListWRange"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7',
                       'test.fail4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail5', 'test.fail6', 'test.fail7', 'test.fail8',
                       'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_includes_list_w_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeIncludesListWRange"
        should_pass = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7',
                       'test.fail4']
        should_fail = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail5', 'test.fail6', 'test.fail7', 'test.fail8',
                       'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)


import pytest

from checkov.common.output.common import compare_table_items_severity


def test_compare_cve_items_severity():
    # given
    cve = [
        {"id": "CVE-2016-6186", "severity": "medium", "fixed_version": "1.8.14"},
        {"id": "CVE-2016-7401", "severity": "high", "fixed_version": "1.8.15"},
        {"id": "CVE-2021-33203", "severity": "medium", "fixed_version": "2.2.24"},
        {"id": "CVE-2019-19844", "severity": "critical", "fixed_version": "1.11.27"},
    ]

    # when
    cve.sort(key=compare_table_items_severity, reverse=True)

    # then
    assert cve == [
        {"id": "CVE-2019-19844", "severity": "critical", "fixed_version": "1.11.27"},
        {"id": "CVE-2016-7401", "severity": "high", "fixed_version": "1.8.15"},
        {"id": "CVE-2016-6186", "severity": "medium", "fixed_version": "1.8.14"},
        {"id": "CVE-2021-33203", "severity": "medium", "fixed_version": "2.2.24"},
    ]


def test_compare_iac_items_severity():
    # given
    iac = [
        {"id": "BC_K8S_1", "severity": "medium"},
        {"id": "BC_K8S_2", "severity": "high"},
        {"id": "BC_K8S_3", "severity": "medium"},
        {"id": "BC_K8S_4", "severity": "critical"},
    ]

    # when
    iac.sort(key=compare_table_items_severity, reverse=True)

    # then
    assert iac == [
        {"id": "BC_K8S_4", "severity": "critical"},
        {"id": "BC_K8S_2", "severity": "high"},
        {"id": "BC_K8S_1", "severity": "medium"},
        {"id": "BC_K8S_3", "severity": "medium"}
    ]



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestNotEqualsIgnoreCaseSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestNotEqualsIgnoreCaseSolver, self).setUp()

    def test_not_equals_ignore_case_solver_wildcard(self):
        root_folder = '../../../resources/encryption_test'
        check_id = "EncryptedResources"
        should_pass = ['aws_rds_cluster.rds_cluster_encrypted', 'aws_s3_bucket.encrypted_bucket',
                       'aws_neptune_cluster.encrypted_neptune']
        should_fail = ['aws_rds_cluster.rds_cluster_unencrypted', 'aws_s3_bucket.unencrypted_bucket',
                       'aws_neptune_cluster.unencrypted_neptune']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        super(TestNotEqualsIgnoreCaseSolver, self).run_test(root_folder=root_folder, expected_results=expected_results,
                                                            check_id=check_id)

    def test_not_equals_ignore_case_solver_boolean(self):
        root_folder = '../../../resources/boolean_test'
        check_id = "BooleanString"
        should_pass = ['azurerm_storage_account.fail1', 'azurerm_storage_account.fail2',
                       'azurerm_storage_account.fail3']
        should_fail = ['azurerm_storage_account.pass1', 'azurerm_storage_account.pass2']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        super(TestNotEqualsIgnoreCaseSolver, self).run_test(root_folder=root_folder, expected_results=expected_results,
                                                            check_id=check_id)



import os
import tempfile
import unittest
from pathlib import Path
from unittest import mock
from unittest.mock import patch
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import CheckType, Report
from checkov.runner_filter import RunnerFilter
from checkov.helm.runner import Runner, fix_report_paths, _get_chart_remote_repos, _get_blocked_helm_repos
from tests.helm.utils import helm_exists


class TestRunnerValid(unittest.TestCase):
    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_record_relative_path_with_relative_dir(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "infrastructure")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace("\\", "/")

        checks_allowlist = ["CKV_K8S_42"]

        runner = Runner()
        report = runner.run(
            root_folder=dir_rel_path, runner_filter=RunnerFilter(framework=["helm"], checks=checks_allowlist)
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.check_type, CheckType.HELM)
        for record in all_checks:
            self.assertIn(record.repo_file_path, record.file_path)
        for resource in report.resources:
            self.assertIn('/helm-tiller/pwnchart/templates', resource)

    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "infrastructure")

        runner = Runner()
        filter = RunnerFilter(framework=['helm'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.HELM: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=scan_dir_path, runner_filter=filter
        )

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        
    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_runner_invalid_chart(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "schema-registry")

        runner = Runner()
        filter = RunnerFilter(framework=['helm'], use_enforcement_rules=False)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        report = runner.run(
            root_folder=scan_dir_path, runner_filter=filter
        )

        self.assertEqual(len(report.failed_checks), 0)

    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_get_binary_output_from_directory_equals_to_get_binary_result(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "schema-registry")

        runner_filter = RunnerFilter(framework=['helm'], use_enforcement_rules=False)

        chart_meta = Runner.parse_helm_chart_details(scan_dir_path)
        chart_item = (scan_dir_path, chart_meta)
        regular_result = Runner.get_binary_output(chart_item, target_dir='./tmp', helm_command="helm",
                                                  runner_filter=runner_filter)
        result_from_directory = Runner.get_binary_output_from_directory(str(scan_dir_path),
                                                                        target_dir='./tmp', helm_command="helm",
                                                                        runner_filter=runner_filter)
        assert regular_result == result_from_directory

    def test_fix_report_paths(self):
        # Create a test report with some checks
        report = Report(CheckType.HELM)
        tmp_dir = "/tmp/helm_test"
        original_root_folder = "/original/root"

        # Create template mapping
        template_mapping = {
            "/tmp/helm_test/manifest1.yaml": "/original/root/chart/templates/manifest1.yaml",
            "/tmp/helm_test/manifest2.yaml": "/original/root/chart/templates/manifest2.yaml",
            "/tmp/helm_test/unknown.yaml": "/original/root/chart/templates/unknown.yaml",
        }

        # Create some test records
        failed_check1 = Record(
            check_id="CKV_K8S_1",
            check_name="Test check 1",
            check_result={"result": CheckResult.FAILED},
            code_block=[],
            file_path=f"{tmp_dir}/manifest1.yaml",
            file_line_range=[1, 10],
            resource="resource1",
            evaluations={},
            check_class="",
            file_abs_path=f"{tmp_dir}/manifest1.yaml",
            entity_tags={},
        )

        passed_check1 = Record(
            check_id="CKV_K8S_2",
            check_name="Test check 2",
            check_result={"result": CheckResult.PASSED},
            code_block=[],
            file_path=f"{tmp_dir}/manifest2.yaml",
            file_line_range=[1, 10],
            resource="resource2",
            evaluations={},
            check_class="",
            file_abs_path=f"{tmp_dir}/manifest2.yaml",
            entity_tags={},
        )

        # Add unknown path check to test edge case
        unknown_check = Record(
            check_id="CKV_K8S_3",
            check_name="Test check 3",
            check_result={"result": CheckResult.FAILED},
            code_block=[],
            file_path=f"{tmp_dir}/unknown.yaml",
            file_line_range=[1, 10],
            resource="resource3",
            evaluations={},
            check_class="",
            file_abs_path=f"{tmp_dir}/unknown.yaml",
            entity_tags={},
        )

        report.failed_checks = [failed_check1, unknown_check]
        report.passed_checks = [passed_check1]

        # Add resources to report
        report.resources = {
            f"{tmp_dir}/manifest1.yaml:resource1",
            f"{tmp_dir}/manifest2.yaml:resource2",
            f"{tmp_dir}/unknown.yaml:resource3"
        }

        # Run the function to test
        fix_report_paths(report, tmp_dir, template_mapping, original_root_folder)

        # Check the results
        self.assertEqual(failed_check1.repo_file_path, "/chart/templates/manifest1.yaml")
        self.assertEqual(failed_check1.file_path, "/chart/templates/manifest1.yaml")
        self.assertEqual(failed_check1.file_abs_path, "/original/root/chart/templates/manifest1.yaml")

        self.assertEqual(passed_check1.repo_file_path, "/chart/templates/manifest2.yaml")
        self.assertEqual(passed_check1.file_path, "/chart/templates/manifest2.yaml")
        self.assertEqual(passed_check1.file_abs_path, "/original/root/chart/templates/manifest2.yaml")

        # Unknown path should just have the temp dir prefix removed
        self.assertEqual(unknown_check.repo_file_path, "/chart/templates/unknown.yaml")

        # Check that resources are also updated
        self.assertIn("/original/root/chart/templates/manifest1.yaml:resource1", report.resources)
        self.assertIn("/original/root/chart/templates/manifest2.yaml:resource2", report.resources)
        self.assertIn("/original/root/chart/templates/unknown.yaml:resource3", report.resources)

    def test_parse_output(self):
        # Create a temp directory for the test
        with tempfile.TemporaryDirectory() as target_dir:
            # Sample helm template output with multiple resources
            helm_output = b"---\n# Source: mychart/templates/service.yaml\napiVersion: v1\nkind: Service\nmetadata:\n  name: example-service\nspec:\n  selector:\n    app: example\n  ports:\n    - port: 80\n      targetPort: 8080\n---\n# Source: mychart/templates/deployment.yaml\napiVersion: apps/v1\nkind: Deployment\nmetadata:\n  name: example-deployment\nspec:\n  replicas: 3\n  template:\n    metadata:\n      labels:\n        app: example\n    spec:\n      containers:\n      - name: example\n        image: example:1.0"

            # Create a temporary chart directory
            with tempfile.TemporaryDirectory() as chart_dir:
                # Set up the chart directory structure
                templates_dir = os.path.join(chart_dir, "templates")
                os.makedirs(templates_dir, exist_ok=True)

                # Create template files to test mapping
                with open(os.path.join(templates_dir, "service.yaml"), 'w') as f:
                    f.write("# Original service template")

                with open(os.path.join(templates_dir, "deployment.yaml"), 'w') as f:
                    f.write("# Original deployment template")

                # Create an empty template mapping dictionary
                template_mapping = {}

                # Call the parse_output function
                Runner._parse_output(target_dir, helm_output, chart_dir, template_mapping)

                # Check template mapping was populated correctly
                expected_mapping = {
                    f'{target_dir}/mychart/templates/service.yaml': os.path.join(chart_dir, "templates/service.yaml"),
                    f'{target_dir}/mychart/templates/deployment.yaml': os.path.join(chart_dir, "templates/deployment.yaml")
                }

                # Compare the mappings - normalize paths for comparison
                normalized_template_mapping = {k.replace('\\', '/'): v.replace('\\', '/')
                                               for k, v in template_mapping.items()}
                normalized_expected_mapping = {k.replace('\\', '/'): v.replace('\\', '/')
                                               for k, v in expected_mapping.items()}

                self.assertEqual(normalized_template_mapping, normalized_expected_mapping)

                # Verify file content was written correctly
                service_file_path = os.path.join(target_dir, "mychart/templates/service.yaml")
                deployment_file_path = os.path.join(target_dir, "mychart/templates/deployment.yaml")

                if os.path.exists(service_file_path):
                    with open(service_file_path, 'r') as f:
                        content = f.read()
                        self.assertIn("kind: Service", content)
                        self.assertIn("name: example-service", content)

                if os.path.exists(deployment_file_path):
                    with open(deployment_file_path, 'r') as f:
                        content = f.read()
                        self.assertIn("kind: Deployment", content)
                        self.assertIn("name: example-deployment", content)

    @unittest.skipIf(not helm_exists(), "helm not installed")
    def test_check_system_deps(self):
        o = Runner().check_system_deps()
        assert o == None
        r = Runner()
        r.helm_command = 'thisshouldfail'
        assert r.check_system_deps() == "helm"

class TestHelmDependencyRemoteRepos(unittest.TestCase):
    """Tests for helm remote dependency repository allowlist.

    Default behaviour: helm template --dependency-update runs normally (original behaviour preserved).
    Blocking is opt-in via CHECKOV_HELM_ALLOWED_REMOTE_REPOS — an allowlist of trusted repo URL prefixes.
    Dependency repos NOT in the allowlist cause the build to be skipped.
    """

    _REMOTE_DEP_DIR = str(Path(__file__).parent / "runner/resources/example_remote_dep")

    def _make_chart_item(self) -> tuple[str, dict]:
        return (self._REMOTE_DEP_DIR, {"name": "example-chart", "version": "0.1.0"})

    # ------------------------------------------------------------------
    # _get_chart_remote_repos() unit tests
    # ------------------------------------------------------------------

    def test_get_chart_remote_repos_detects_http_url(self):
        """_get_chart_remote_repos() must return http:// dependency repos."""
        repos = _get_chart_remote_repos(self._REMOTE_DEP_DIR)
        self.assertIn("http://192.0.2.1", repos)

    def test_get_chart_remote_repos_returns_empty_for_missing_dir(self):
        """_get_chart_remote_repos() must not raise for a missing directory."""
        self.assertEqual(_get_chart_remote_repos("/nonexistent/path"), [])

    # ------------------------------------------------------------------
    # _get_blocked_helm_repos() unit tests (allowlist logic)
    # ------------------------------------------------------------------

    def test_get_blocked_helm_repos_returns_empty_when_no_env_var(self):
        """Default: no env var → _get_blocked_helm_repos() returns [] (allow all)."""
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_HELM_ALLOWED_REMOTE_REPOS", None)
            repos = _get_blocked_helm_repos(self._REMOTE_DEP_DIR)
        self.assertEqual(repos, [])

    def test_get_blocked_helm_repos_returns_blocked_when_not_in_allowlist(self):
        """Repos NOT in the allowlist must be returned (they will be blocked)."""
        env = {"CHECKOV_HELM_ALLOWED_REMOTE_REPOS": "https://charts.trusted.org/"}
        with mock.patch.dict(os.environ, env):
            repos = _get_blocked_helm_repos(self._REMOTE_DEP_DIR)
        self.assertIn("http://192.0.2.1", repos)

    def test_get_blocked_helm_repos_returns_empty_when_all_in_allowlist(self):
        """When all repos are in the allowlist, nothing is blocked."""
        env = {"CHECKOV_HELM_ALLOWED_REMOTE_REPOS": "http://192.0.2.1"}
        with mock.patch.dict(os.environ, env):
            repos = _get_blocked_helm_repos(self._REMOTE_DEP_DIR)
        self.assertEqual(repos, [])

    # ------------------------------------------------------------------
    # get_binary_output() integration tests
    # ------------------------------------------------------------------

    def test_dependency_update_in_args_by_default(self):
        """Default (no env var): --dependency-update is in helm args (original behaviour)."""
        chart_item = self._make_chart_item()
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        mock_proc = mock.MagicMock()
        mock_proc.communicate.return_value = (fake_output, b"")
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_HELM_ALLOWED_REMOTE_REPOS", None)
            with mock.patch("subprocess.Popen", return_value=mock_proc) as mock_popen:
                Runner.get_binary_output(chart_item, "./tmp", "helm", RunnerFilter())
        call_args = mock_popen.call_args[0][0]
        self.assertIn("--dependency-update", call_args)

    def test_build_skipped_when_repo_not_in_allowlist(self):
        """When CHECKOV_HELM_ALLOWED_REMOTE_REPOS is set and a repo is not in it, build is skipped."""
        chart_item = self._make_chart_item()
        # The first subprocess.Popen call is 'helm dependency list' (before the allowlist check).
        # We must mock it so it does not crash; the allowlist check then returns None before the second call.
        dep_list_proc = mock.MagicMock()
        dep_list_proc.communicate.return_value = (b"", b"")
        env = {"CHECKOV_HELM_ALLOWED_REMOTE_REPOS": "https://charts.trusted.org/"}
        with mock.patch.dict(os.environ, env):
            with mock.patch("subprocess.Popen", return_value=dep_list_proc) as mock_popen:
                result, _ = Runner.get_binary_output(chart_item, "./tmp", "helm", RunnerFilter())
        self.assertIsNone(result)
        # Only the dependency list call should have been made; helm template must NOT be called.
        self.assertEqual(mock_popen.call_count, 1)
        self.assertIn("dependency", mock_popen.call_args[0][0])

    def test_build_proceeds_when_all_repos_in_allowlist(self):
        """When all repos are in the allowlist, build proceeds normally."""
        chart_item = self._make_chart_item()
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        dep_list_proc = mock.MagicMock()
        dep_list_proc.communicate.return_value = (b"", b"")
        template_proc = mock.MagicMock()
        template_proc.communicate.return_value = (fake_output, b"")
        env = {"CHECKOV_HELM_ALLOWED_REMOTE_REPOS": "http://192.0.2.1"}
        with mock.patch.dict(os.environ, env):
            with mock.patch("subprocess.Popen", side_effect=[dep_list_proc, template_proc]) as mock_popen:
                result, _ = Runner.get_binary_output(chart_item, "./tmp", "helm", RunnerFilter())
        self.assertEqual(mock_popen.call_count, 2)
        self.assertEqual(result, fake_output)

    def test_chart_dir_always_last_arg(self):
        """chart_dir must always be the last argument."""
        chart_item = self._make_chart_item()
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        mock_proc = mock.MagicMock()
        mock_proc.communicate.return_value = (fake_output, b"")
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_HELM_ALLOWED_REMOTE_REPOS", None)
            with mock.patch("subprocess.Popen", return_value=mock_proc) as mock_popen:
                Runner.get_binary_output(chart_item, "./tmp", "helm", RunnerFilter())
        call_args = mock_popen.call_args[0][0]
        self.assertEqual(call_args[-1], self._REMOTE_DEP_DIR)


if __name__ == "__main__":
    unittest.main()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.ELBAccessLogs import check
from checkov.common.models.enums import CheckResult


class TestELBAccessLogs(unittest.TestCase):

    def test_failure_elb_1(self):
        hcl_res = hcl2.loads("""
          resource "aws_elb" "test" {
            name = "test-lb-tf"
            availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]

            listener {
              instance_port     = 8000
              instance_protocol = "http"
              lb_port           = 80
              lb_protocol       = "http"
            }
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_elb_2(self):
        hcl_res = hcl2.loads("""
          resource "aws_elb" "test" {
            name = "test-lb-tf"
            availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]

            listener {
              instance_port     = 8000
              instance_protocol = "http"
              lb_port           = 80
              lb_protocol       = "http"
            }

            access_logs {
              bucket  = aws_s3_bucket.lb_logs.bucket
              enabled = false   
            }
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)           

    def test_success_elb_1(self):
        hcl_res = hcl2.loads("""
        resource "aws_elb" "test" {
            name = "test-lb-tf"
            availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]

            listener {
              instance_port     = 8000
              instance_protocol = "http"
              lb_port           = 80
              lb_protocol       = "http"
            }

            access_logs {
              bucket  = aws_s3_bucket.lb_logs.bucket
              enabled = true
            }
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_elb_2(self):
        hcl_res = hcl2.loads("""
        resource "aws_elb" "test" {
            name = "test-lb-tf"
            availability_zones = ["us-west-2a", "us-west-2b", "us-west-2c"]

            listener {
              instance_port     = 8000
              instance_protocol = "http"
              lb_port           = 80
              lb_protocol       = "http"
            }

            access_logs {
              bucket  = aws_s3_bucket.lb_logs.bucket
              # The default value for enabled is true              
            }
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_elb']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)          


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
import re
from typing import Any

from checkov.common.util.type_forcers import force_int
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SecretManagerSecret90days(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Secrets Manager secrets should be rotated within 90 days"
        id = "CKV_AWS_304"
        supported_resources = ("aws_secretsmanager_secret_rotation",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def _check_rate_expression(self, expression: str) -> bool:
        rate_match = re.match(r'rate\((\d+)\s+(days?|hours?|minutes?)\)', expression)
        if rate_match:
            value = int(rate_match.group(1))
            unit = rate_match.group(2)

            if unit.startswith('day'):
                return value <= 90
            elif unit.startswith('hour'):
                return value <= 2160  # 90 days * 24 hours
            elif unit.startswith('minute'):
                return value <= 129600  # 90 days * 24 hours * 60 minutes
        return False

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["rotation_rules"]
        rules = conf.get("rotation_rules")
        if rules and isinstance(rules, list):
            rotation_rule = rules[0]

            # Check for automatically_after_days
            days = rotation_rule.get('automatically_after_days')
            if days and isinstance(days, list):
                self.evaluated_keys = ["rotation_rules/[0]/automatically_after_days"]
                days = force_int(days[0])
                if days is not None and days <= 90:
                    return CheckResult.PASSED

            # Check for schedule_expression
            schedule = rotation_rule.get('schedule_expression')
            if schedule and isinstance(schedule, list):
                self.evaluated_keys = ["rotation_rules/[0]/schedule_expression"]
                expression = schedule[0]

                if expression.startswith('rate('):
                    return CheckResult.PASSED if self._check_rate_expression(expression) else CheckResult.FAILED
                elif expression.startswith('cron('):
                    # TODO: Handle failing cron expressions
                    return CheckResult.PASSED

        return CheckResult.FAILED


check = SecretManagerSecret90days()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.AzureDefenderOnKubernetes import check
from checkov.common.models.enums import CheckResult


class TestAzureDefenderOnKubernetes(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_security_center_subscription_pricing" "example" {
                  tier          = "Free"
                  resource_type = "KubernetesService"
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_security_center_subscription_pricing']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_security_center_subscription_pricing" "example" {
                  tier          = "Standard"
                  resource_type = "KubernetesService"
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_security_center_subscription_pricing']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.kubernetes.checks.resource.k8s.k8s_check_utils import extract_commands


def test_non_int_extract_commands() -> None:
    conf = {'command': ['kube-apiserver', '--encryption-provider-config=config.file']}

    keys, values = extract_commands(conf)
    assert keys == ['kube-apiserver', '--encryption-provider-config']
    assert values == ['', 'config.file']


def test_int_extract_commands() -> None:
    conf = {'command': ['kube-apiserver', '--encryption-provider-config=config.file', '-p', 9082]}

    keys, values = extract_commands(conf)
    assert keys == ['kube-apiserver', '--encryption-provider-config', '-p', 9082]
    assert values == ['', 'config.file', '', '']



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestIsEmptySolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestIsEmptySolver, self).setUp()

    def test_is_empty_solver_simple(self):
        root_folder = './'
        check_id = "SGPorts"
        should_pass = ['aws_security_group.sg3']
        should_fail = ['aws_security_group.aws_security_group_public', 'aws_security_group.sg2']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_is_empty_solver_jsonpath(self):
        root_folder = './'
        check_id = "SGPortsJsonpath"
        should_pass = ['aws_security_group.sg3']
        should_fail = ['aws_security_group.aws_security_group_public', 'aws_security_group.sg2']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.SecretExpirationDate import check
from checkov.common.models.enums import CheckResult


class TestSecretExpirationDate(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault_secret" "example" {
              name         = "secret-sauce"
              value        = "szechuan"
              key_vault_id = azurerm_key_vault.example.id
            
              tags = {
                environment = "Production"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault_secret']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault_secret" "example" {
              name         = "secret-sauce"
              value        = "szechuan"
              key_vault_id = azurerm_key_vault.example.id
            
              tags = {
                environment = "Production"
              }
              expiration_date = "2020-12-30T20:00:00Z"
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault_secret']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



"""Tests for the in-memory verified-sources loader and meta-path finder.

Covers Phase 2 of the verification module:

* Direct load via ``load_verified_sources_into_module`` execs the
  in-memory bytes, not the on-disk file (verify-then-load consistency
  for direct loads).
* ``VerifiedSourcesFinder``, while installed, serves an ``import``
  request from the in-memory map (verify-then-load consistency for
  transitive imports like ``import _helper`` from inside a verified
  check).
* ``BaseCheckRegistry.load_external_checks`` with ``verified_sources``
  refuses any on-disk ``.py`` file that is absent from the allowlist.
* The finder is fully cleaned up from ``sys.meta_path`` after
  ``load_external_checks`` returns — no leakage into the rest of the
  Python program.
* The unverified default disk-load path (``verified_sources=None``) is
  byte-for-byte unchanged. This is the supported path when no public
  key is configured — see ``base_check_registry._load_external_checks_from_disk``.
"""
from __future__ import annotations

import sys
from pathlib import Path

import pytest

from checkov.common.external_checks.verification.verified_loader import (
    VerifiedSourcesFinder,
    install_finder,
    load_verified_sources_into_module,
    uninstall_finder,
)


@pytest.fixture(autouse=True)
def _restore_sys_state():
    """Snapshot+restore sys.modules / sys.meta_path / sys.path around each test.

    Without this, a failing test could leave the meta-path finder
    installed and break the rest of the suite, or leave a half-loaded
    module in ``sys.modules`` that masks a real import error in the
    next test.
    """
    saved_modules = set(sys.modules.keys())
    saved_meta_path = list(sys.meta_path)
    saved_path = list(sys.path)
    try:
        yield
    finally:
        for added in set(sys.modules.keys()) - saved_modules:
            sys.modules.pop(added, None)
        sys.meta_path[:] = saved_meta_path
        sys.path[:] = saved_path


def test_load_verified_sources_into_module_execs_in_memory_bytes(tmp_path: Path):
    """Bytes argument is what's executed, even when the disk file differs."""
    on_disk = tmp_path / "module_a.py"
    on_disk.write_bytes(b"VALUE = 'disk'\n")
    in_memory_bytes = b"VALUE = 'memory'\nFLAG = 42\n"

    module = load_verified_sources_into_module(
        "module_a_test_in_memory", str(on_disk), in_memory_bytes,
    )

    assert module.VALUE == "memory"  # in-memory bytes won
    assert module.FLAG == 42
    assert module.__file__ == str(on_disk)


def test_load_verified_sources_into_module_writes_no_bytecode_cache(tmp_path: Path):
    """The loader must NOT write a ``.pyc`` next to the source.

    A persistent bytecode cache on disk is a second representation of the
    module that does not carry a trailer of its own. Disabling cache writes
    keeps the executed-bytes / verified-bytes equality strictly per-process
    and avoids leaving artefacts in customer directories.
    """
    on_disk = tmp_path / "no_cache_target.py"
    on_disk.write_bytes(b"VALUE = 'verified'\n")

    load_verified_sources_into_module(
        "no_cache_target_mod", str(on_disk), b"VALUE = 'verified'\n",
    )

    # No __pycache__ directory created next to the source.
    assert not (tmp_path / "__pycache__").exists(), (
        f"unexpected __pycache__ in {tmp_path}: "
        f"{list((tmp_path / '__pycache__').iterdir()) if (tmp_path / '__pycache__').exists() else []}"
    )
    # And no stray .pyc anywhere in tmp_path.
    pyc_files = list(tmp_path.rglob("*.pyc"))
    assert pyc_files == [], f"unexpected .pyc files written: {pyc_files}"


def test_load_verified_sources_into_module_cleans_up_on_failure(tmp_path: Path):
    """A SyntaxError in the bytes leaves no half-loaded module behind."""
    on_disk = tmp_path / "broken.py"
    on_disk.write_bytes(b"# empty\n")

    with pytest.raises(SyntaxError):
        load_verified_sources_into_module(
            "module_a_test_broken",
            str(on_disk),
            b"def broken(:\n",  # SyntaxError
        )

    assert "module_a_test_broken" not in sys.modules


def test_verified_source_loader_exec_module_cleans_up_on_failure(tmp_path: Path):
    """``_VerifiedSourceLoader.exec_module`` must mirror the direct loader's
    half-load cleanup — a SyntaxError or in-module raise during the finder
    path must NOT leave a partially initialised module in ``sys.modules``.

    Pins S6: without the cleanup, a single broken verified file could
    poison subsequent legitimate imports of the same name with a
    half-initialised module object.
    """
    import importlib.util
    from checkov.common.external_checks.verification.verified_loader import (
        _VerifiedSourceLoader,
    )

    module_name = "broken_via_finder_exec"
    loader = _VerifiedSourceLoader(
        module_name, str(tmp_path / "broken.py"), b"def broken(:\n",
    )
    spec = importlib.util.spec_from_loader(module_name, loader)
    assert spec is not None
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    try:
        with pytest.raises(SyntaxError):
            loader.exec_module(module)
        assert module_name not in sys.modules, (
            f"{module_name} survived a SyntaxError during exec_module; "
            f"future imports of this name would receive the half-initialised "
            f"module object instead of a clean ModuleNotFoundError"
        )
    finally:
        sys.modules.pop(module_name, None)


def test_finder_resolves_imports_from_in_memory_map(tmp_path: Path):
    """A normal ``import`` call goes through the finder when active."""
    on_disk = tmp_path / "helper_module.py"
    on_disk.write_bytes(b"raise RuntimeError('disk path was executed')\n")
    in_memory_bytes = b"VALUE = 'from-map'\n"

    finder = install_finder(
        {str(on_disk): in_memory_bytes},
        [str(tmp_path)],
    )
    try:
        module_name = "helper_module_test_in_memory"
        # The finder maps the import name 'helper_module' (via filename
        # 'helper_module.py') to the in-memory bytes; we use a uniquified
        # alias to avoid colliding with another test's sys.modules entry.
        import importlib
        spec = finder.find_spec("helper_module")
        assert spec is not None
        module = importlib.util.module_from_spec(spec)
        sys.modules[module_name] = module
        spec.loader.exec_module(module)
        assert module.VALUE == "from-map"  # in-memory bytes, not disk
    finally:
        uninstall_finder(finder)


def test_finder_returns_none_for_unknown_names(tmp_path: Path):
    """Unrecognised import names fall through to the next finder."""
    finder = VerifiedSourcesFinder({}, [str(tmp_path)])
    assert finder.find_spec("os") is None
    assert finder.find_spec("no_such_module_anywhere") is None
    assert finder.find_spec("pkg.submodule") is None  # dotted names ignored


def test_finder_is_removed_from_sys_meta_path_after_uninstall(tmp_path: Path):
    """install_finder + uninstall_finder leaves sys.meta_path as it was.

    Uses identity-based comparison (``id()``) rather than ``==`` /
    ``in``. List equality and the ``in`` operator both invoke ``__eq__``
    on every element of ``sys.meta_path``; on Python 3.9 inside xdist
    workers we have observed third-party MetaPathFinder instances whose
    ``__eq__`` can hang indefinitely (waiting on a lock held by an
    xdist control thread). Identity comparison avoids the user-defined
    equality path entirely while still verifying the contract: the
    finder we installed is gone, and the rest of the meta-path is the
    same set of objects in the same order.
    """
    before_ids = [id(x) for x in sys.meta_path]
    finder = install_finder({}, [str(tmp_path)])
    assert id(finder) in [id(x) for x in sys.meta_path]
    uninstall_finder(finder)
    after_ids = [id(x) for x in sys.meta_path]
    assert id(finder) not in after_ids
    assert after_ids == before_ids


def test_uninstall_finder_is_idempotent(tmp_path: Path):
    """Calling uninstall twice does not raise."""
    finder = install_finder({}, [str(tmp_path)])
    uninstall_finder(finder)
    uninstall_finder(finder)  # second call must be a no-op


def test_resolve_verified_source_only_honours_canonical_keys(tmp_path: Path, stub_registry):
    """``_resolve_verified_source`` returns None for non-canonical keys.

    Pins S1: the lookup is documented as "registry keys are realpath",
    full stop. A raw-path fallback (``verified_sources.get(canonical)
    or verified_sources.get(raw)``) would silently honour any dict key
    that exactly matched the requested ``check_full_path``, even when
    that key was never realpath-normalised by the registry. Such keys
    cannot exist in the production registry (per
    ``sources_registry.verify_and_register``), so a fallback is dead
    code at best — and a footgun if a future caller mutates the
    in-memory map with non-canonical keys.

    To distinguish a raw key from its canonical form we point a symlink
    at a real file and key the dict by the symlinked path. The lookup
    canonicalises via ``realpath`` (resolves the symlink) and so the
    raw symlinked key MUST NOT match.
    """
    import os

    real = tmp_path / "real_check.py"
    real.write_bytes(b"# real signed body\n")
    link = tmp_path / "alias_check.py"
    try:
        os.symlink(real, link)
    except (OSError, NotImplementedError):
        pytest.skip("symlinks not supported on this platform")

    registry = stub_registry

    # Key the dict by the SYMLINKED (raw) path — the registry would
    # never have produced this key because verify_and_register
    # realpath-normalises every key.
    raw_link_path = str(link)
    verified_sources = {raw_link_path: b"unverified-bytes-the-registry-never-issued"}

    result = registry._resolve_verified_source(verified_sources, raw_link_path)

    assert result is None, (
        "non-canonical (symlinked) key lookup must return None — the "
        "raw-path fallback is removed; if this fails the fallback has "
        "been reintroduced and S1 has regressed"
    )

    # Sanity: when the dict IS keyed by the realpath, the lookup finds it.
    canonical_path = os.path.normpath(os.path.realpath(raw_link_path))
    canonical_sources = {canonical_path: b"verified-canonical-bytes"}
    assert (
        registry._resolve_verified_source(canonical_sources, raw_link_path)
        == b"verified-canonical-bytes"
    )


def test_load_external_checks_with_verified_sources_runs_in_memory(
    tmp_path: Path, capsys, stub_registry,
):
    """The registry loader execs the allowlist bytes, ignoring disk content."""
    checks_dir = tmp_path / "checks"
    checks_dir.mkdir()
    (checks_dir / "__init__.py").write_bytes(b"")
    on_disk_check = checks_dir / "verified_check.py"
    on_disk_check.write_bytes(b"print('FROM_DISK')\n")  # would be visible if exec'd
    in_memory_bytes = b"print('FROM_MEMORY')\n"

    verified_sources = {
        str(on_disk_check.resolve()): in_memory_bytes,
    }

    stub_registry.load_external_checks(
        str(checks_dir.resolve()), verified_sources=verified_sources,
    )

    captured = capsys.readouterr()
    assert "FROM_MEMORY" in captured.out
    assert "FROM_DISK" not in captured.out


def test_load_external_checks_refuses_unverified_file(
    tmp_path: Path, caplog, stub_registry,
):
    """A .py file present on disk but absent from verified_sources is REFUSED and ESCALATES.

    Two-part contract:
    1. The bytes are NEVER exec'd — refused at the resolve step.
    2. The loader raises ``SignatureVerificationError`` at the end of the
       walk so the chokepoint can route the failure through
       ``_report_verification_failure_and_exit`` and exit 2.

    Part 2 closes the M1/S3 disk-drift gap: an empty allowlist or a
    late-added on-disk file (rename / git stash pop / build-script
    artefact / unrelated process write) must not silently shrink the
    verified check set.
    """
    import logging
    from checkov.common.external_checks.verification import (
        SignatureVerificationError,
    )

    checks_dir = tmp_path / "checks"
    checks_dir.mkdir()
    (checks_dir / "__init__.py").write_bytes(b"")
    extra = checks_dir / "unverified_extra.py"
    # Bytes that would *raise* if executed — proves the loader did not exec.
    extra.write_bytes(b"raise RuntimeError('unverified file was executed')\n")

    with caplog.at_level(logging.ERROR), pytest.raises(SignatureVerificationError) as exc:
        stub_registry.load_external_checks(
            str(checks_dir.resolve()),
            verified_sources={},  # empty allowlist → every .py is unverified
        )

    # The escalation diagnostic must name the offending file so the
    # operator sees exactly what diverged on disk.
    assert "unverified_extra.py" in str(exc.value)
    # And the per-file ERROR log line remains as a UX nicety.
    assert any(
        "Refusing to load unverified external check" in record.message
        for record in caplog.records
    )


def test_load_external_checks_cleans_up_finder(tmp_path: Path, stub_registry):
    """sys.meta_path returns to its pre-call state once load returns."""
    checks_dir = tmp_path / "checks"
    checks_dir.mkdir()
    (checks_dir / "__init__.py").write_bytes(b"")
    chk = checks_dir / "simple.py"
    chk.write_bytes(b"X = 1\n")

    before = list(sys.meta_path)
    stub_registry.load_external_checks(
        str(checks_dir.resolve()),
        verified_sources={str(chk.resolve()): b"X = 1\n"},
    )
    after = list(sys.meta_path)

    # No VerifiedSourcesFinder may remain on sys.meta_path.
    assert not any(isinstance(f, VerifiedSourcesFinder) for f in after)
    # And the path stack must be unchanged in identity-of-finders too.
    assert len(after) == len(before)


def test_unverified_load_external_checks_unchanged_when_no_verified_sources(
    tmp_path: Path, capsys, stub_registry,
):
    """Default behaviour (no verified_sources) execs from disk as before.

    This is the supported path for operators who don't pass
    ``--external-checks-public-key``; it must remain byte-identical to
    pre-feature behaviour so existing customers see no change.
    """
    checks_dir = tmp_path / "checks"
    checks_dir.mkdir()
    (checks_dir / "__init__.py").write_bytes(b"")
    chk = checks_dir / "unverified_check.py"
    chk.write_bytes(b"print('UNVERIFIED_DISK_LOAD')\n")

    stub_registry.load_external_checks(str(checks_dir.resolve()))  # verified_sources=None

    captured = capsys.readouterr()
    assert "UNVERIFIED_DISK_LOAD" in captured.out



from pathlib import Path

import pytest
from pytest_mock import MockerFixture

from checkov.kubernetes.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.graph_utils.utils import GRAPH_FRAMEWORKS

RESOURCES_DIR = Path(__file__).parent / "resources"


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_runner(mocker: MockerFixture, graph_framework):
    # given
    test_file_path = RESOURCES_DIR / "graph_check.yaml"
    runner_filter = RunnerFilter(checks=["CKV2_K8S_6"])

    mocker.patch.dict("os.environ", {"CHECKOV_GRAPH_FRAMEWORK": graph_framework})

    # when
    report = Runner().run(root_folder=None, files=[str(test_file_path)], runner_filter=runner_filter)

    #  when
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 1
    assert summary["skipped"] == 1
    assert summary["parsing_errors"] == 0



from pathlib import Path

from checkov.azure_pipelines.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.runner_filter import RunnerFilter
from checkov.azure_pipelines.checks.registry import registry

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_registry_has_type():
    assert registry.report_type == CheckType.AZURE_PIPELINES


def test_runner_honors_enforcement_rules():
    # given
    test_file = EXAMPLES_DIR / "azure-pipelines.yml"

    # when
    filter = RunnerFilter(framework=["azure_pipelines"], use_enforcement_rules=True)
    # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
    # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
    filter.enforcement_rule_configs = {CheckType.AZURE_PIPELINES: Severities[BcSeverities.OFF]}
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=filter)

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


def test_runner_passing_check():
    # given
    test_file = EXAMPLES_DIR / "azure-pipelines.yml"

    # when
    report = Runner().run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZUREPIPELINES_1"])
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


def test_runner_failing_check():
    # given
    test_file = EXAMPLES_DIR / "azure-pipelines.yml"

    # when
    report = Runner().run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_AZUREPIPELINES_2"])
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0
